/**
 * @copyright University of Guelph
 */

#ifndef _BLOCK_H_
#define _BLOCK_H_ 1

#include <assert.h>
#include <stdio.h>

#include <string>
#include <vector>

#include "yaml.h"

namespace starplacer {
    class Net;

    class Block {
        friend class Net;

      public:
       double x, y; // FPGA coordinate

       typedef std::vector<Net*>::iterator net_iter;

      private:
        const int id_; // block_id same as vpr's id
        // can be used as an index for lookups
        // [0 .. placement.netlist.num_blocks]

        std::vector<Net*> nets_;
        // the nets connected to this block.

      public:

        /**
         * Create a block with
         * @param num_pins the number of pins connected to it.
         * each pin may or may not connect to a net.
         * @param id a unique id for this block
         */
        Block(int num_pins, int id);

        virtual ~Block() {}

        inline net_iter nets_begin() {
            return nets_.begin();
        }

        inline net_iter nets_end() {
            return nets_.end();
        }

        inline int id() const {
            return id_;
        }

        inline size_t num_nets() {
            return nets_.size();
        }

        inline Net* nets(int i) {
            assert(i >= 0 && i < (int)nets_.size());
            return nets_[i];
        }

        /* dumps the block attributes to the screen */
        virtual void print() const;

        /**
         * shortens the nets array by removing NULL
         * entries.
         * the specific pin which a net is on doesn't
         * matter for placement, it is only used in routing
         * so condensing the list of nets by removing nulls
         * avoids having to check in for loops for null
         * values and skipping them over.
         *
         * this should only be called after all nets have been added.
         */
        void CondenseEdges();

        /**
         * Set the net on the pin
         * this net can later be retrieved
         * by calling nets(pin) unless
         * CondenseEdges has been called
         */
        virtual void addNet(Net* net, int pin);
    };
}


namespace yaml {
    /** get a yaml version of the block */
    std::string yaml(starplacer::Block* b);
}

#endif
