/**
 * @copyright University of Guelph
 */

#ifndef __CORNER_PREPLACER_H__
#define __CORNER_PREPLACER_H__ 1

#include "Block.h"
#include "SuccessiveOverRelaxation.h"
#include "Placement.h"

namespace starplacer {
    class CornerPrePlacer {
      public:
        Block* topleft;
        Block* topright;
        Block* bottomleft;
        Block* bottomright;

        /**
         * The user only needs to call the constructor with an instance of SOR
         * to use this class, this pre-placement method requires SOR because of
         * its ordering of variables for evaluation is also based on closeness.
         * This pre-placer chooses four blocks which are not closely connected
         * to each other. It does this by setting all blocks to one extreme and
         * the block to compare against at the other extreme and then using SOR
         * to update the block positions and the block which is farthest away from the
         * comparison block is the one which is most loosely connected.
         * The results are stored in the public member variables. All other blocks
         * are placed at the centre of the FPGA.
         */
        CornerPrePlacer(SuccessiveOverRelaxation& sor);

      private:
        Placement* placement;
        Block* GetMaxXBlock();
    };
}

#endif // __CORNER_PREPLACER_H__
