/**
 * @copyright University of Guelph
 */

#ifndef _PLACEMENT_H_
#define _PLACEMENT_H_ 1

#include <cstddef>
#include <assert.h>

#include <vector>
#include <list>
#include <fstream>
#include <string>

#include "yaml.h"
#include "IOBlock.h"
#include "Block.h"
#include "Net.h"
#include "Architecture.h"
#include "Netlist.h"

namespace starplacer {
    class Placement {
      public:
        typedef std::vector<Block*>::iterator block_iterator;
        typedef std::vector<IOBlock*>::iterator io_iterator;
        typedef std::vector<Net*>::iterator net_iterator;
        typedef std::list<Block*>::iterator isolated_iterator;

        Netlist& netlist;
        Architecture& arch;
        std::vector<Net*> nets; // nets indexed by net.id()
        std::vector<Block*> blocks; // both io and clbs
        std::vector<IOBlock*> io_blocks; // just ios
        std::vector<Block*> clb_blocks; // just clbs
        std::list<Block*> isolated_blocks; // not connected to any other blocks

      private:
        std::vector<double> stored_x;
        std::vector<double> stored_y;

      public:

        Placement(Netlist& netlist, Architecture& arch);

        ~Placement();

        /**
         * Computes the Current Star+ Wirelength cost for the placement.
         * @return the Star+ cost for the entire placement.
         */
        double StarPlusWirelengthCost();

        /**
         * store the current solution,
         * note only stores one solution, two calls result in the older being overwritten
         * note: the Z coordinates of the IO blocks are not currently stored.
         */
        void Store();

        /**
         * Set the placement to be what it was when store was called.
         */
        void Restore();

        /**
         * writes the CLB coordinates (x and y) to file
         * in a tab separated format
         * this is the output of the program.
         */
        void WriteToFile(const std::string& filename) const;
    };
} // namespace starplacer


namespace yaml {
    std::string yaml(starplacer::Placement* p);
}

#endif

