/**
 * @file Arch.h
 * Defines a class layer/interface between the VPR globals and Starplacer.
 * @copyright University Of Guelph
 * @author Ming Xu
 */


#ifndef __SetPartition_H__
#define __SetPartition_H__ 1

namespace starplacer {
    class SetPartition {
      private:
        SetPartition* parent;  // the parent node
        int size;
        int cost;

        /**
         * Makes this SetPartition the child of the root node
         * and decrements the cost for every level between the root and SetPartition
         */
        void optimize();

        /**
         * Returns the set (root of the tree) which uniquely identifies
         * this partition
         */
        SetPartition* getSet() const;

      public:
        SetPartition();

        /**
         *
         * Combines the two set partition trees, in effect making them
         * the same partition, all nodes in either tree now belong
         * to the same partition
         */
        static void CombinePartitions(SetPartition *c, SetPartition *b);

        /**
         * Test whether two nodes belong to the same tree or partition
         */
        bool isSamePartition(const SetPartition& B);

        bool operator==(const SetPartition& B);
        /**
         * @return the root of the tree.
         */
    };
}

#endif /*__SetPartition_H__*/
