#ifndef __YAML_H__
#define __YAML_H__ 1

#include <sstream>
#include <string>
#include <iterator>
#include <vector>
#include <list>

/**
 * This file is a very light weight, limited, and
 * somewhat hack YAML serialization.
 * it does not handle nested lists as this this would
 * lead to a recursive template
 * however, it is easy to extend by adding new
 * overloaded ``yaml()` functions for user-defined types.
 */

namespace yaml {
    inline std::string yaml(int x) {
        std::stringstream ss;
        ss << x;
        return ss.str();
    }

    inline std::string yaml(double x) {
        std::stringstream ss;
        ss << x;
        return ss.str();
    }

    inline std::string yaml(std::string s) {
        std::stringstream ss;
        ss << "\"" << s << "\"";
        return ss.str();
    }

    inline std::string yaml(float x) {
        std::stringstream ss;
        ss << x;
        return ss.str();
    }

    template<class Iterator>
    std::string yaml_list(Iterator begin, Iterator end) {
        std::stringstream ss;
        ss << "[";
        while (begin != end) {
            ss << yaml(*begin);
            if (++begin != end) {
                ss << ", ";
            }
        }
        ss << "]";
        return ss.str();
    }

    template<typename T>
    std::string yaml(std::vector<T>& vec) {
        return yaml_list(vec.begin(), vec.end());
    }

    template<typename T>
    std::string yaml(std::list<T>& vec) {
        return yaml_list(vec.begin(), vec.end());
    }

    inline std::string yaml(std::vector<int> vi) {
        return yaml_list(vi.begin(), vi.end());
    }

} // namespace yaml

#endif // __YAML_H__
