/**
 * @copyright University of Guelph
 */

#include <assert.h>
#include <stdio.h>
#include <cstddef>

#include <vector>
#include <sstream>
#include <string>

#include "Block.h"
#include "Net.h"
#include "yaml.h"

namespace starplacer {
    typedef std::vector<Net*>::iterator net_iter;

    Block::Block(int num_pins, int id): id_(id) {
        assert(id_ >= 0);
        x = 0;
        y = 0;
        nets_.resize(num_pins, NULL);
    }

    void Block::print() const {
        printf("id:%d x: %.2f, y:%.2f\n", id_, x, y);
    }

    void Block::addNet(Net* net, int blk_pin) {
        assert(net);
        nets_[blk_pin] = net;
    }

    void Block::CondenseEdges() {
        int open = 0;
        for (size_t i = 0; i < nets_.size(); ++i) {
            if (nets_[i]) {
                nets_[open++] = nets_[i];
            }
        }
        nets_.resize(open);
    }
}

namespace yaml {
    std::string yaml(starplacer::Block* b) {
        std::stringstream stream;
        if (b == NULL) {
            return "null";
        }
        stream << "{id: " << b->id();
        stream << ", x: " << b->x;
        stream << ", y: " << b->y;
        stream << ", type: \"CLBBlock\"}";
        return stream.str();
    }
}
