/**
 * @copyright University of Guelph
 */

#include <cstddef>
#include <math.h>
#include <assert.h>
#include <stdio.h>

#include <algorithm>
#include <limits>
#include <vector>

#include "yaml.h"
#include "SuccessiveOverRelaxation.h"
#include "Block.h"
#include "IOBlock.h"
#include "Net.h"
#include "CornerPrePlacer.h"
#include "SimpleBipartition.h"
#include "Bipartition.h"

#include "SORPlacer.h"

namespace starplacer {
       void SORPlacer::Place(Placement* placement) {
        SuccessiveOverRelaxation sor(placement);
        CornerPrePlacer corner(sor);

        sor.QuickConverge();

        // add the corner pieces
        sor.AddToSequence(corner.topleft);
        sor.AddToSequence(corner.topright);
        sor.AddToSequence(corner.bottomleft);
        sor.AddToSequence(corner.bottomright);

        // update the nets to include the corners
        sor.UpdateNets();

        sor.RemoveNets();

        // Find the ios and clbs which are not isolated !
        std::vector<Block*>not_iso_clbs;
        std::vector<IOBlock*>not_iso_ios;

        not_iso_ios.reserve(placement->io_blocks.size());
        not_iso_clbs.reserve(placement->clb_blocks.size());

        for (Placement::io_iterator bi = placement->io_blocks.begin();
                bi != placement->io_blocks.end(); ++bi) {
            if ((*bi)->num_nets() > 0) {
                    not_iso_ios.push_back(*bi);
            }
        }

        for (Placement::block_iterator bi = placement->clb_blocks.begin();
            bi != placement->clb_blocks.end(); ++bi) {
            for (size_t i = 0; i < (*bi)->num_nets(); ++i) {
                if ((*bi)->nets(i)->is_global == false) {
                    not_iso_clbs.push_back(*bi);
                    break;
                }
            }
        }

        bool use_bipart = false;

        printf("---#  Star+ Wire-length Cost\n");
        SimpleBipartition legalize_A(placement, 1);
        Bipartition legalize_B(0, placement->arch.ny + 1, 0, placement->arch.nx + 1, placement->arch.io_rat);

        double cost;
        int iter_count = 0;
        int count = std::max(std::max(placement->arch.nx, placement->arch.ny), 2);
        while (count > 1) {
            sor.ConvergeBlock(count);
            if (use_bipart) {
                legalize_B.clear();
                legalize_B.LoadPoints(not_iso_clbs.begin(), not_iso_clbs.end(), not_iso_ios.begin(), not_iso_ios.end());
                legalize_B.Partition();
            } else {
                legalize_A.Initialize(not_iso_clbs, not_iso_ios);
                legalize_A.Partition();
            }
            cost = sor.cost();
            printf(" -  %d\n", (int)cost);
            count = (int)(count *0.9);
            iter_count++;
        }

        legalize_A.Initialize(not_iso_clbs, not_iso_ios);
        legalize_A.SpreadIO();
        printf("Iterations: %d\n\n", iter_count);
    }
}
