/**
 * @copyright University of Guelph
 */

#include <assert.h>
#include <cstddef>

#include "SetPartition.h"

namespace starplacer {
    SetPartition::SetPartition() {
        size = 1;
        cost = 0;
        parent = NULL;
    }

    void SetPartition::CombinePartitions(SetPartition *c, SetPartition*self) {
        c = c->getSet();
        self = self->getSet();

        if (c == self) {
            return;
        }

        if (self->size + c->cost > self->cost + c->size) {
            c->parent = self;
            self->size += c->size;
            self->cost += c->size - 1;
        } else {
            self->parent = c;
            c->size += self->size;
            c->cost += self->size - 1;
        }
    }

    void SetPartition::optimize() {
        if (parent->parent == NULL) {
            return;
        }

        SetPartition* root = parent->parent;

        while (root->parent != NULL) {
            root = root->parent;
        }

        SetPartition* p = this;
        SetPartition* pp = parent;

        while (pp != root) {
            p->parent = root;
            p = pp;
            pp = p->parent;
            root->cost--;
        }
    }

    SetPartition* SetPartition::getSet() const {
        if (parent == NULL) {
            return const_cast<SetPartition*>(this);
        }

       const_cast<SetPartition*>(this)->optimize();

        return parent;
    }

    bool SetPartition::isSamePartition(const SetPartition& B) {
        return B.getSet() == getSet();
    }

    bool SetPartition::operator==(const SetPartition& B) {
        return isSamePartition(B);
    }
}
